(function($, DropperFramework) {

// only run once!
if (window.kkDropZones) return;

/****************************************************************
 * kkDropZones Class
 * 
 * Displays Drop Zones live on the page and helps to select them
 ****************************************************************/
var kkDropZones = function() {
	var self = this;
	$(function() {
		self.init();
	});
};

kkDropZones.prototype = {
	
	/**
	 * Initializes this class instance
	 */
	init: function() {
		// build up the dom
		var self = this;
		
		// selection result
		self.result = {
			selector: null,
			method: null
		};
		
		// initial mouse
		self.mouse = {
			x: -1,
			y: -1
		};

		// area where zone markers are placed into
		self.jMarkerArea = $('<div class="kk-dropzone-markers" style="display:none"></div>');

		// create the marker
		this.jMarker = $(
			'<div class="kk-dropzone-marker">\
				<div class="kk-dropzone-method-marker kk-dropzone-insert-line" data-method="before"></div>\
				<div class="kk-dropzone-marker-margin">\
					<div class="kk-dropzone-marker-padding">\
						<div class="kk-dropzone-method-marker kk-dropzone-insert-line" data-method="prepend"></div>\
						<div class="kk-dropzone-marker-content"></div>\
						<div class="kk-dropzone-method-marker" data-method="html"></div>\
						<div class="kk-dropzone-method-marker kk-dropzone-insert-line" data-method="append"></div>\
					</div>\
				</div>\
				<div class="kk-dropzone-method-marker kk-dropzone-insert-line" data-method="after"></div>\
				<div class="kk-dropzone-method-marker" data-method="replaceWith"></div>\
			</div>');		

		this.jMarkerArea.append(this.jMarker);

		// select the current zone on click. mark all zones below it as well
		this.jMarker.click(function(e) {
			if(!self.pickedZone) { 
				self.pickZone(self.markedZone, e.pageX, e.pageY, true);
			}
		});

		// hover bubble
		self.jPopover = $('\
			<div class="kk-dropzone-popover kk-dropzone-popover-initializing" style="display:none">\
				<div class="kk-dropzone-popover-arrow"></div>\
				<div class="kk-dropzone-results">\
					<div class="kk-dropzone-popover-result main-result"></div>\
					<div class="kk-dropzone-popover-more-results" style="display:none"></div>\
					<div class="kk-dropzone-methods" style="display:none">\
						<div class="kk-dropzone-method-info">Einf&uuml;gemodus</div>\
						<div class="kk-dropzone-method" data-method="before" data-method-label="Vorstellen"></div>\
						<div class="kk-dropzone-method" data-method="after" data-method-label="Anstellen"></div>\
						<div class="kk-dropzone-method" data-method="prepend" data-method-label="Vorh&auml;ngen"></div>\
						<div class="kk-dropzone-method" data-method="append" data-method-label="Anh&auml;ngen"></div>\
						<div class="kk-dropzone-method" data-method="html" data-method-label="Inhalt ersetzen"></div>\
						<div class="kk-dropzone-method" data-method="replaceWith" data-method-label="Ersetzen"></div>\
					</div>\
				</div>\
			</div>');


		// backdrop
		self.jBackdrop = $('<div class="kk-dropzone-markers-backdrop"></div>');

		// wire them together
		self.jPopover.appendTo(self.jMarkerArea);
		self.jMarkerArea.prepend(self.jBackdrop);

		// append the marker area to the end of the body
		self.jMarkerArea.appendTo('body');

		// stop propagation when the marker info bubble is clicked
		self.jPopover.click(function(e){
			e.stopPropagation();
		});

		// select all drop zones and init
		self.initZones();

		// no picked zone yet
		self.pickedZone = null;

		// set ourself active by default
		self.setActive(true);

		// remove zone selection when the backdrop is clicked
		self.jBackdrop.click(function(e){
			e.stopPropagation();
			self.pickZone(null, null, null, true);
		});

		// update marker info overlay on mouse move
		$('body').mousemove(function(e) {
			self.mouse = {
				x: e.pageX,
				y: e.pageY
			};
			
			if (!self.pickedZone) self.updatePopoverPosition(e.pageX, e.pageY);
		});

		// avoid middle clicking on a link opens a new tab
		$('body').on('click', 'a', function(e) {
			if(e.which === 2) {
				e.preventDefault();
				e.stopPropagation();
			}
		});

		// handle mouse down events.
		// middle button toggles mode
		$('body').mousedown(function(e) {

			if(e.which === 2) {

				// since this toggles in any case, we never want the middle button to open a link
				e.preventDefault();
				e.stopPropagation();

				// avoid annoying double middle clicks (maybe only a linux problem)
				var diff = self.lastMiddleClick ? e.timeStamp - self.lastMiddleClick : 99999;
				self.lastMiddleClick = e.timeStamp;

				if (diff > 300) {
					self.toggleActive();
				}
			}
		});

		// handle key events
		$(window).keydown(function(e) {
			if($(document.activeElement).is(':focus')) return;
			// esc key toggles overlay mode
			if(e.keyCode == 27) {
				if(self.pickedZone){ 
					self.pickZone(null, null, null, true);
				}
				else {
					self.toggleActive();
				}
				e.preventDefault();
				e.stopPropagation();
			}
			// ctrl + space toggles
			if(e.ctrlKey === true && e.keyCode === 32) {
				self.toggleActive();
			}
		});

		// handle parent window message events
		this.handleParentMessages();

		// check for result hover and act
		self.handleResultInteractions();
		
		// whenever a ajax call was made we update the whole marker area

		var ajaxUpdateTimeout = null;
		var ajaxUpdate = function() {
			clearTimeout(ajaxUpdateTimeout);
			ajaxUpdateTimeout = setTimeout(function() {
				self.initZones();
			}, 1000);
		};
		
		// watch for usual ajax calls
		$(document).ajaxSuccess(ajaxUpdate);
		// watch for xajax calls
		$(function() {
			DropperFramework.hook('init_article', ajaxUpdate);
		});

		// override jquery ajax
		self.overrideJqueryAjax();
		
		// start
		self.update();
	},

	update: function() {
		var self = this;

		clearTimeout(self.updateTimeout);

		self.updateTimeout = setTimeout(function() {

			window.cancelAnimationFrame(self.updateAnimationFrame);
			self.updateAnimationFrame = window.requestAnimationFrame(function() {

				if (self.active) {
			
					// set marker area size to document boundaries
					self.jMarkerArea.css({
						width: '',
						height: ''
					});
					
					self.jMarkerArea.css({
						width: $(document).width(),
						height: $(document).height()
					});
			
					// keep the marker in sync with the hovered zone
					if (!self.pickedZone) {
						var info = self.getZonesAtPosition(self.mouse.x, self.mouse.y, true);
						if (info) {
							self.jPopover.css('display', '');
							// has the hovered zone changed?
							if (!self.markedZone || (self.markedZone && info.zone.get(0) !== self.markedZone.get(0))) {
								//console.log('hover zone changed', info.zone, self.markedZone);
								self.setPopoverInfo(info.zone);
								self.updateMarker(info.zone);
							}
						} else {
							// no zone found, hide the picker
							self.jPopover.css('display', 'none');
						}
					}
					
					// update the marker frequently to reflect the zones measures
					if (self.markedZone) self.updateMarker();
				}
				
				// run us again at the next frame
				self.update();
			});
			
		}, 75);
	},
	
	/**
	 * When the page makes an ajax call the php side
	 * wants to know if we are in picker mode.
	 * we are overriding the jQuery ajax method here and
	 * enhancing the data array
	 */
	overrideJqueryAjax: function() {
		var ajaxFn = $.ajax;

		$.ajax = function() {
			var settings = null;

			if (arguments.length == 1) {
				settings = arguments[0];
			} else if (arguments.length == 2) {
				settings = arguments[1];
			}

			if (settings) {
				$.extend(true, settings, {
					data: {
						zonePicker: true
					}
				});
			}
			return ajaxFn.apply($, arguments);
		};
	},
	
	/**
	 * finds all zones in the DOM and removes all known markers and the references 
	 * on thier zones. this is needed after ajax requests
	 */
	initZones: function() {
		
		var self = this;

		// find new zones
		self.jZones = $('.kk-dropzone-ghost-begin[data-ghost-dropzone], [data-dropzone]');

		// enhance zone data, find virtual ghost zones
		for (var i=0; i<self.jZones.length; i++) {

			var jZone = $(self.jZones[i]);
			
			// calcualate z-index
			if (!jZone.data('zIndex')) { 
				jZone.data(
					'zIndex', 
					1000*self.getZIndex(jZone) + self.getDOMDepth(jZone) + i/1000
				);
			}

			var jGhostBegin = jZone;

			// no further tweaks when this is not a ghostzone
			if (!jGhostBegin.is('.kk-dropzone-ghost-begin[data-ghost-dropzone]')) continue;

			var jGhostEnd = jGhostBegin.nextAll('.kk-dropzone-ghost-end[data-ghost-dropzone="' + jGhostBegin.attr('data-ghost-dropzone') + '"]:first');

			if (!jGhostEnd.length) { 
				console.warn('no end node found for ', jGhostBegin.get(0));
				continue; 
			}

			var jGhostZone = $('<div style="display:none" data-dropzone="' + jGhostBegin.attr('data-ghost-dropzone') + '" data-dropzone-all="(' + jGhostBegin.attr('data-ghost-dropzone') + ')" data-dropzone-name="' + jGhostBegin.attr('data-ghost-dropzone-name') + '" data-dropzone-info="' + jGhostBegin.attr('data-ghost-dropzone-info') + '" data-dropzone-type="' + jGhostBegin.attr('data-ghost-dropzone-type') + '"></div>');

			// set some special data for this zone and insert it into the DOM
			var beginNode = jGhostBegin.get(0);
			var endNode = jGhostEnd.get(0);
			var parentNode = beginNode.parentNode;
			
			if (!beginNode || !endNode || beginNode === endNode || beginNode.nextSibling === endNode || endNode.previousSibling === beginNode) {
				// skip
				console.log('empty ghost zone detected, skipping:', jGhostBegin.attr('data-ghost-dropzone-name'));
				continue;
			}

			jGhostZone.data('zIndex', jZone.data('zIndex'));

			jGhostZone.data('isGhost', true);
			jGhostZone.data('parentNode', parentNode);

			jGhostZone.data('startsAfterNode', beginNode.previousSibling && beginNode.previousSibling.nodeName.toLowerCase() === 'meta' ? beginNode.previousSibling.previousSibling : beginNode.previousSibling);
			jGhostBegin.remove();

			jGhostZone.data('endsBeforeNode', endNode.nextSibling && endNode.nextSibling.nodeName.toLowerCase() === 'meta' ? endNode.nextSibling.nextSibling : endNode.nextSibling);
			jGhostEnd.remove();

			// create a ghost zone placeholder
			jGhostZone.appendTo('body');

			// add the ghost zone to our overall zone results
			self.jZones[i] = jGhostZone;

		}

		// sort results by z-index
		self.jZones.sort(function(a, b) {
			return ($(a).data('zIndex') - $(b).data('zIndex'));
		});

		// remove all ghost markers
		$('[data-ghost-dropzone]').remove();

	},

	/**
	 * In the set of matched zone info elements, this will return the first visible one
	 */
	getFirstVisibleZoneInfo: function(infos) {
		for (var i=0; i<infos.length; i++) {
			if (this.isZoneVisible(infos[i].zone)) return infos[i];
		}
		return null;
	},


	/**
	 * Gets information about all zones intersecting the given position
	 */
	getZonesAtPosition: function(x, y, returnFirst) {
		var self = this;
		var result = [];

		for (var i=self.jZones.length-1; i>=0; i--) {
			var jZone = $(self.jZones[i]);
			var rect = self.getBoundingBox(jZone);

			if (x >= rect.left && x <= rect.right && y >= rect.top && y <= rect.bottom) { 
				var info = {
					zone: jZone,
					rect: rect
				};

				if (returnFirst) return info;

				result.push(info);
			}
		}

		
		if (returnFirst && !result.length) return null;

		return result;
	},

	/**
	 * Gets the bounding box from a set of given DOM elements
	 */
	getBoundingBox: function(jZone) {
		var isGhost = jZone.data('isGhost');
		var nodes = isGhost ? this.getGhostZoneNodes(jZone) : [jZone.get(0)];
		var invalid = { top: 0, left: 0, right: 0, bottom: 0, width: 0, height: 0 };

		// not valid or empty?
		if (!nodes || !nodes.length) return invalid;

		var top=false, left=false, right=false, bottom=false;

		for (var i=0; i<nodes.length; i++) {

			var node = nodes[i];
			var rects = [];
			
			// Element node - use jQuery to measure
			if (node.nodeType === 1) {
				var jNode = $(node);
				if (!this.isZoneVisible(jNode)) continue;
				var pos = jNode.offset();
				pos.left -= window.pageXOffset;
				pos.top -= window.pageYOffset;
				rects = [{ top: pos.top, left: pos.left, right: pos.left + jNode.outerWidth(false), bottom: pos.top + jNode.outerHeight(false) }];
			} 
			// Text nodes - cannot measure them with jQuery, take a hardcore approach
			else if (node.nodeType === 3) {
				var range = document.createRange();
				range.selectNodeContents(node);
				rects = range.getClientRects();
			}

			if (rects.length) {
				for (var j=0; j<rects.length; j++) {
					var rect = rects[j];

					top = top === false ? rect.top : Math.min(top, rect.top);
					left = left === false ? rect.left : Math.min(left, rect.left);
					right = right === false ? rect.right : Math.max(right, rect.right);
					bottom = bottom === false ? rect.bottom : Math.max(bottom, rect.bottom);
				}
			}
		}
		
		// invalid?
		if (top === false || left === false || bottom === false || right === false) return invalid;

		// create final result
		var res = {
			top: top + window.pageYOffset,
			left: left + window.pageXOffset,
			right: right + window.pageXOffset,
			bottom: bottom + window.pageYOffset,
			width: right - left,
			height: bottom - top,
			zIndex: 0 //this.getZIndex(isGhost && nodes[0].nodeType === 3 ? nodes[0].parentNode : nodes[0])
		};

		return res;
	},
	
	/**
	 * gets a list of DOM-Nodes for the given ghost zone
	 */
	getGhostZoneNodes: function(jGhostZone) {
		// get previous starting node
		var startsAfterNode = jGhostZone.data('startsAfterNode');
		var endsBeforeNode = jGhostZone.data('endsBeforeNode');
		var parentNode = jGhostZone.data('parentNode');

		var startNode, endNode;
		
		// if we have no parent this is invalid
		if (!parentNode || !parentNode.childNodes.length) { 
			console.log('Ghost Zone parent was not given or has no children... skipping:', jGhostZone.attr('data-dropzone-name'));
			return [];
		}
		
		// when startsAfterNode is null we take the first child
		startNode = startsAfterNode && startsAfterNode.nextSibling 
						? startsAfterNode.nextSibling
						: parentNode.firstChild;


		// when endsBeforeNode is null we take the last child
		endNode = endsBeforeNode && endsBeforeNode.previousSibling 
					? endsBeforeNode.previousSibling
					: parentNode.lastChild;

		// if between startnode and endnode is no further sibling let's simply skip
		if (!startNode || !endNode) {
			console.log('Empty ghost zone... skipping:', jGhostZone.attr('data-dropzone-name'), startNode, endNode);
			return [];
		}

		// our nodes collection
		var nodes = [];
		
		// loop all sibling nodes inbetween startsAfterNode and endsBeforeNode
		var node = startNode;

		do {
			if (node.nodeType === 3 || (node.nodeType === 1 && ['script', 'style', 'meta'].indexOf(node.tagName.toLowerCase()) < 0)) nodes.push(node);
			if (node === endNode) break;
			node = node.nextSibling;
		} while (node);

		return nodes;
	},

	/**
	 * activates or deactivates the zone overlay
	 */
	setActive: function(state, skipHostNotification) {
		this.active = state;
		if(this.active) {
			this.jMarkerArea.css('display', '');
			this.updatePopoverPosition();
			$('body').addClass('kk-dropzone-picker-active');
		} else {
			this.jMarkerArea.css('display', 'none');
			$('body').removeClass('kk-dropzone-picker-active');
		}
		// notify our host window about this change
		if (!skipHostNotification && window.parent && window.parent.postMessage) {
			window.parent.postMessage({
				type: 'kk_dropper.selectionMode',
				active: this.active
			}, '*');
		}
	},

	/**
	 * Toggles the active state of the zone overlay
	 */
	toggleActive: function() {
		this.setActive(!this.active);
	},

	/**
	 * Listen if our parent Window wants to change the selection mode
	 * and react accordingly
	 */
	handleParentMessages: function() {
		var self = this;
		window.onmessage = function(msg) {
			if (!msg || !msg.data || !msg.data.type) return;
			// selection mode change?
			if (msg.data.type === 'kk_dropper.selectionMode') {
				self.setActive(msg.data.active, true);
			}
			// preselect zone?
			if (msg.data.type === 'kk_dropper.presetResult') {
				$(function() {
					if (msg.data.zone && msg.data.zone.sampleFn) {
						// if we have a sample function, eval it
						eval(msg.data.zone.sampleFn);
						setTimeout(function() {
							self.preview(msg.data.selector, msg.data.replaceMethod || '');
						}, 500);
					} else {
						self.preview(msg.data.selector, msg.data.replaceMethod || '');
					}
				});
			}
		};
	},

	/**
	 * shows all zones at the same time or hides them
	 */
	showAllZones: function(state) {
		if(state === true) {
			this.setActive(true);
			this.jMarkerArea.find('.kk-dropzone-marker').addClass('kk-dropzone-marker-active');
		} else {
			this.jMarkerArea.find('.kk-dropzone-marker').removeClass('kk-dropzone-marker-active');
		}
	},

	/**
	 * Refreshes the marker of a given zone
	 * @jZone - the zone DOM element, jQuery wrapped
	 */
	updateMarker: function(jZone) {
		var self = this;

		if (!jZone || !jZone.length) { 
			jZone = self.markedZone;
		} else {
			self.markedZone = jZone;
		}
		
		if (!jZone || !jZone.length) return;
		
		var jMarkerMargin = self.jMarker.find('.kk-dropzone-marker-margin');
		var jMarkerPadding = self.jMarker.find('.kk-dropzone-marker-padding');

		// ghost zone is simple
		if (jZone.data('isGhost')) {
			var rect = self.getBoundingBox(jZone);
			jMarkerPadding.css('border-width', '0');
			jMarkerMargin.css('border-width', '0');
			self.jMarker.css({
				'top': rect.top,
				'left': rect.left,
				'width': rect.width,
				'height': rect.height
			});
		}
		// no ghost zone? display margin and paddings
		else {
			var pos = jZone.offset();
			jMarkerPadding.css({
				'border-top-width': jZone.css('padding-top'),
				'border-right-width': jZone.css('padding-right'),
				'border-bottom-width': jZone.css('padding-bottom'),
				'border-left-width': jZone.css('padding-left')
			});
	
			jMarkerMargin.css({
				'border-top-width': Math.max(0, parseFloat(jZone.css('margin-top'))),
				'border-right-width': Math.max(0, parseFloat(jZone.css('margin-right'))),
				'border-bottom-width': Math.max(0, parseFloat(jZone.css('margin-bottom'))),
				'border-left-width': Math.max(0, parseFloat(jZone.css('margin-left')))
			});

			self.jMarker.css({
				'top': pos.top - Math.max(0, parseFloat(jZone.css('margin-top'))),
				'left': pos.left - Math.max(0, parseFloat(jZone.css('margin-left'))),
				'width': jZone.outerWidth() + Math.max(0, parseFloat(jZone.css('margin-left'))) + Math.max(0, parseFloat(jZone.css('margin-right'))),
				'height': jZone.outerHeight() + Math.max(0, parseFloat(jZone.css('margin-top'))) + Math.max(0, parseFloat(jZone.css('margin-bottom')))
			});

		}
		
	},

	/**
	 * Checks if the given element is visible inside the DOM tree
	 */
	isZoneVisible: function(jZone) {

		// check if not visible
		if (!jZone || !jZone.length) return false;
		if (jZone.data('isGhost')) {
			var nodes = this.getGhostZoneNodes(jZone);
			if (!nodes || !nodes.length) return false;
			jZone = $(nodes[0].parentNode);
		}
		if (!jZone.is(':visible')) return false;
		var vis = jZone.css('visibility');
		if (vis === 'hidden' || vis === 'collapse') return false;
		if (jZone.css('opacity') == 0) return false;

		// no more parent?
		if (!jZone.parent() || jZone.parent().is('body') || jZone.parent().is('html')) return true;

		// return parent visibility
		return this.isZoneVisible(jZone.parent());
	},

	/**
	 * gets the absolute z-index of a given DOM Element
	 * @el - the dom element to get the z-index for
	 */
	getZIndex: function(el) {
		var elem = $(el).get(0);
		if(elem === document || elem === window) return 0;
		var z = +window.getComputedStyle(elem, null).getPropertyValue('z-index');
		if (isNaN(z)) return this.getZIndex(elem.parentNode);
		return z; 
	},

	/**
	 * calculates how deep an element is nested inside the DOM
	 */
	getDOMDepth: function(el) {
		var depth = 0;
		var elem = $(el).get(0);
		while (elem && elem !== window && elem.parentNode) {
			depth ++;
			elem = elem.parentNode;
		}
		return depth;
	},

	/**
	 * Fills the Information popover with the markers info
	 * @jMarker - the jquery wrapped marker element
	 */
	setPopoverInfo: function(jZone){
		
		if (!jZone || !jZone.length) return;

		var jContent = '\
			<div class="kk-dropzone-popover-info">' + jZone.attr('data-dropzone-info') + '</div>\
			<div class="kk-dropzone-popover-label">' + jZone.attr('data-dropzone-name') + '</div>\
			';
			
		this.jPopover.find('.main-result').html(jContent);
		this.jPopover.attr('data-zone-type', jZone.attr('data-dropzone-type'));
		this.jPopover.attr('data-zone-ghost', jZone.data('isGhost') ? 'true' : 'false');
		this.jPopover.css('display', '');

	},

	/**
	 * Updates the tooltip position
	 */
	_updateRequest: 0,
	
	updatePopoverPosition: function(x, y) {

		var self = this;
		
		if (self._updateRequest) window.cancelAnimationFrame(self._updateRequest);

		self._updateRequest = window.requestAnimationFrame(function() {

			if(!$.isNumeric(x)) x = self.jPopover.data('x') || 0;
			if(!$.isNumeric(y)) y = self.jPopover.data('y') || 0;
	
			self.jPopover.removeClass('kk-dropzone-popover-initializing');

			self.jPopover.data('x', x);
			self.jPopover.data('y', y);


			// reset old position
			self.jPopover.css({
				left: '',
				top: '',
				right: '',
				bottom: ''
			});

			self.jPopover.removeClass('hdir-left hdir-center hdir-right vdir-top vdir-bottom');
	
			// calc direction
			var vDir = 'top';


			// calc and keep in bounds
			var popoverWidth = self.jPopover.outerWidth(true);
			var popoverHeight = self.jPopover.outerHeight(true);
			var documentWidth = $(document).width();
			var documentHeight = $(document).height();

			var bounds = {
				left: x - popoverWidth/2,
				top: y,
				right: x + popoverWidth/2,
				bottom: y + popoverHeight
			};

			if (y + Math.max(popoverHeight, 450) > documentHeight) vDir = 'bottom';
			
			// set classes
			self.jPopover.addClass('vdir-' + vDir);

			bounds.left = Math.max(0, bounds.left);
			bounds.left = Math.min(documentWidth - popoverWidth, bounds.left);
	
			// set horizontal dir
			self.jPopover.css('left', bounds.left + 'px');
	
			// set vertical dir
			if (vDir === 'top') self.jPopover.css('top', bounds.top + 'px');
			if (vDir === 'bottom') self.jPopover.css('bottom', documentHeight - y);

		});
	},
	
	/**
	 * Handle hovering and clicking of a picker result
	 */
	handleResultInteractions: function() {
		var self = this;
		
		// result click
		self.jPopover.on('click', '.kk-dropzone-popover-result', function(e) {
			if (!window.parent || !window.parent.postMessage) return;

			// the zone bound to this result
			var jZone = $(this).data('zone');
			self.setResultZone(jZone, true);
		});

		// result hover
		self.jPopover.on('mouseenter', '.kk-dropzone-popover-result', function(e) {
			// set the text inside the mail result
			if (!self.result.jZone) {
				var jResult = $(this);
				var jZone = jResult.data('zone');
				
				// set popover attributes to this zone
				self.jPopover.attr('data-zone-type', jZone.attr('data-dropzone-type'));
				self.jPopover.attr('data-zone-ghost', jZone.data('isGhost') ? 'true' : 'false');
	
				self.pickedZone = jZone;

				self.setPopoverInfo(jZone);
				self.updateMarker(jZone);
				self.updatePopoverPosition();
			}
		});
		
		// method selection hover and unhover (show insert markers)
		self.jPopover.on('mouseenter', '.kk-dropzone-method', function(e) {
			if (!self.result.jZone) return;
			self.setResultMethod(self.result.jZone, $(this).attr('data-method'));
		});

		self.jPopover.on('mouseleave', '.kk-dropzone-method', function(e) {
			if (!self.result.jZone) return;
			self.setResultMethod(self.result.jZone, false);
		});

		// method selection click
		self.jPopover.on('click', '.kk-dropzone-method', function(e) {
			e.stopPropagation();
			
			var jMethod = $(this);
			var method = jMethod.attr('data-method');
			self.result.method = method;
			self.sendResult(self.result);
			self.previewMethod = method;
			self.setResultMethod(self.result.jZone, self.result.method);
		});
	},


	/**
	 * sets the result zone, given opening the method chooser
	 */
	setResultZone: function(jZone, animate) {
		var self = this;
		
		// set popover info to this zone
		this.setPopoverInfo(jZone);

		// set result selector
		self.result.selector = jZone.attr('data-dropzone');
		self.result.jZone = jZone;
		
		// show method chooser
		self.showMethodChooser(animate);
	},
	
	/**
	 * selects the result method
	 */
	setResultMethod: function(jZone, method) {

		var map = {
			'prepend': 'Vorh&auml;ngen',
			'append': 'Anh&auml;ngen',
			'after': 'Anstellen',
			'before': 'Vorstellen',
			'html': 'Inhalt ersetzen',
			'replaceWith': 'Ersetzen'
		};

		this.jMarkerArea.find('.kk-dropzone-method-marker.method-visible').removeClass('method-visible');
		this.jPopover.find('.kk-dropzone-method-selected').removeClass('kk-dropzone-method-selected');

		var setMethod = method || this.previewMethod;
		
		if (setMethod) {
			this.jMarker.find('.kk-dropzone-method-marker[data-method="' + setMethod + '"]').addClass('method-visible');
		}
		
		this.jPopover.find('.kk-dropzone-method-info').html(setMethod ? map[setMethod] : 'Einf&uuml;gemethode');
		
		// set preview method selected?
		if (this.previewMethod) {
			this.jPopover.find('.kk-dropzone-method[data-method="' + this.previewMethod + '"]').addClass('kk-dropzone-method-selected');
		}
	},

	/**
	 * Displays the method chooser and hides the picker results
	 */
	showMethodChooser: function(animate) {
		var self = this;
		self.jMarkerArea.addClass('pick-mode-method');

		self.jPopover.find('.kk-dropzone-popover-more-results').slideUp(animate ? 200 : 0, function(){
			// reset min-width restrictions
			self.jPopover.css('min-width', '');
			self.updatePopoverPosition();
			self.jPopover.find('.kk-dropzone-methods').slideDown(animate ? 200 : 0, function() {
				self.updatePopoverPosition();
			});
		});
	},

	/**
	 * Sends the picker result to the embedding iframe parent window
	 */
	sendResult: function(result) {
		var sendResult = {
			type: 'kk_dropper.pickerResult', 
			selector: result.selector, 
			method: result.method 
		};

		console.log('sending picker result', sendResult);

		if (window.parent && window.parent.postMessage) {
			window.parent.postMessage(sendResult, '*');
		}
	},
	
	pickZone: function(jZone, x, y, animate) {

		var self = this;

		// exit pick mode?
		if (!jZone) {
			self.exitPickMode();
		}
		// enable pick mode and enhance the info with parent zones
		else if (jZone && jZone.length) {
			// set the main result
			this.jPopover.find('.main-result').data('zone', jZone);
			
			// we do not want to have weird width-jumping effects on the popover, set a min-width
			this.jPopover.css('min-width', this.jPopover.outerWidth(false));

			// set the main results text
			this.setPopoverInfo(jZone);

			var zones = this.getZonesAtPosition(x, y);
			var jMore = self.jPopover.find('.kk-dropzone-popover-more-results');

			jMore.empty();
			jMore.css('display', 'none');

			// append further results down the line
			if (zones.length > 1) {
				var skipCSSZones = false;

				for(var i=1; i<Math.min(zones.length, 10); i++){
					var resZone = zones[i].zone;
					var zoneType = resZone.attr('data-dropzone-type');

					// whenever we reach a "zone" prevent showing other selectors
					if (['zone'].indexOf(zoneType) >= 0) skipCSSZones = true;
					
					// skip css zones
					if (skipCSSZones && ['drop', 'zone'].indexOf(zoneType) < 0) { 
						continue;
					}
					
					var jResult = $('\
						<div class="kk-dropzone-popover-result kk-dropzone-popover-result-' + resZone.attr('data-dropzone-type') + '">\
							<div class="kk-dropzone-popover-label" title="' + resZone.attr('data-dropzone-info') + (resZone.attr('data-dropzone-info') ? ' &raquo; ' : '') + resZone.attr('data-dropzone-name') + '">' + resZone.attr('data-dropzone-name') + '</div>\
						</div>'
						);
		
					jResult.data('zone', resZone);
					jMore.append(jResult);
				}
				
				jMore.slideDown(animate ? 'fast' : 0, function() {
					self.updatePopoverPosition();
				});
			}

	
			this.pickedZone = jZone;
			this.jPopover.css('display', '');
			this.jMarkerArea.addClass('pick-mode');
	
			// if we've got coordinates lets update the marker position
			if($.isNumeric(x) && $.isNumeric(y)) { 
				this.updatePopoverPosition(x, y);
			}
		}
	},

	/**
	 * leave the pick mode
	 */
	exitPickMode: function() {
		var self = this;

		self.result.selector = null;
		self.result.method = null;
		self.result.jZone = null;
		self.previewMethod = null;

		self.jPopover.find('.kk-dropzone-methods').slideUp('fast');

		self.jPopover.find('.kk-dropzone-popover-more-results').slideUp('fast', function(){
			self.jPopover.find('.kk-dropzone-results').slideDown('fast');
			self.pickedZone = null;
			self.updatePopoverPosition();
			self.jMarkerArea.removeClass('pick-mode');
			self.jMarkerArea.removeClass('pick-mode-method');
			// reset min-width restrictions
			self.jPopover.css('min-width', '');
		});
		
		self.setResultMethod(null, false);
	},
	
	/**
	 * Displays a selector, method set as if it has been selected by the user
	 */
	preview: function(selector, method) {
		if (!selector || !selector.trim().length) return false;

		var jZone = $('*[data-dropzone-all*="(' + selector + ')"]');
		
		// check if there was no zone found and we have a usual css selector
		if ((!jZone || !jZone.length) && selector.indexOf('@') < 0) {
			console.log('no direct zone found, selecting by css rule', selector);
			// try to select the element
			jZone = $(selector).eq(0);
			if (!jZone.length) return;

			// this should not already be a zone
			if (!jZone.attr('data-dropzone') || jZone.attr('data-dropzone').indexOf('@') < 0) {

				console.log('no zone found to preview, trying to create it (' + selector + ')');

				jZone.attr('data-dropzone', selector);
				jZone.attr('data-dropzone-name', selector);
				jZone.attr('data-dropzone-info', 'CSS Selektor');
				jZone.attr('data-dropzone-type', selector.split(' ').reverse()[0].trim().indexOf('#') === 0 ? 'id' : 'class');
			}
		}
		
		// still no zone or text node selected?
		if (!jZone.length || jZone.get(0).nodeType === 3) return false;

		console.log('zone found:', jZone);

		this.pickZone(jZone, null, null, false);
		this.updateMarker(jZone);
		this.setResultZone(jZone, false);
		
		if (method) {
			this.previewMethod = method;
			this.setResultMethod(jZone, method);
		}

		// center popover
		var coords = this.jMarker.offset();
		
		// move to bottom
		if (method == 'prepend') coords.top;
		if (method == 'before') coords.top -= parseFloat(jZone.css('margin-top'));
		if (method == 'append') coords.top += this.jMarker.outerHeight(false);
		if (method == 'after') coords.top += this.jMarker.outerHeight(false);
		if (method == 'html' || method=='replaceWith') coords.top += this.jMarker.outerHeight(false)/2 + parseFloat(jZone.css('padding-top'));
		this.updatePopoverPosition(coords.left + this.jMarker.outerWidth(false)/2, coords.top);

		// scroll the page to our zone
		$(document).scrollTop(Math.max(0, coords.top - $(window).height()/2));
		
		return true;
	}
};

// show drop zones
window.kkDropZones = new kkDropZones();

})(jQuery, DropperFramework);