'use strict';

/**
 * Route paths that are available
 */
var DROPPER_ROUTES = {

	'backend': '/',
	'expired': '/expired',
	'live': '/live',
	'default': '/'

};


angular.module('controls', []);

angular.module('app', ['ngRoute', 'controls', 'oc.lazyLoad', 'datetime', 'colorpicker.module', angularDragula(angular)]);

angular.module('app').controller('AppCtrl', ['$scope', '$rootScope', '$location', 'rootScopeLegacy', 'core', 'upgrade', '$ocLazyLoad', '$element', '$q', '$timeout', '$route', function($scope, $rootScope, $location, rootScopeLegacy, core, upgrader, $ocLazyLoad, $element, $q, $timeout, $route) {

	$scope.ready = false;

	// extend the rootscope with some legacy functions that we still need to keep old drops running :(
	rootScopeLegacy.$install($rootScope);

	/**
	 * Generates a list of resources and returns a promise
	 * @id - kind of resource
	 * @type - either js or css
	 * @list - array of files to inject
	 */
	var collectResources = function(type, list) {
		var files = [];

		for (var i=0; i<list.length; i++) {
			var url = list[i];
			if (url && url.length) {
				files.push({ type: type, path: url });
			}
		}
		
		return files;
	};
	
	/* 
	 * Injects scripts using jQuery
	 * @files - an array of files: [{ type: "js|css", path: "url/to/file" }, ...]
	 */
	var injectScripts = function(files) {

		// empty list or none? resolve immediately
		if (!files || !files.length) { 
			var defer = $q.defer();
			defer.resolve();
			return defer.promise;
		}

		var promises = [];
		
		$.each(files, function(idx, file) {
			var defer = $q.defer();
			promises.push(defer.promise);
			
			$.ajax({
				url: file.path,
				dataType: 'script',
				scriptCharset: 'utf-8',
				cache: true,
				success: function(res) {
					defer.resolve(res);
				},
				error: function(err) {
					defer.reject(err);
				}
			});
			
		});
		
		return $q.all(promises);
	};
	
	/* 
	 * Lazy loads modules using ocLazyLoad
	 * @files - an array of files: [{ type: "js|css", path: "url/to/file" }, ...]
	 */
	var injectModules = function(files) {

		// empty list or none? resolve immediately
		if (!files || !files.length) { 
			var defer = $q.defer();
			defer.resolve();
			return defer.promise;
		}

		var loaderOpts = [
			{ 
				files: files,
				cache: true, 
				insertBefore: '#content', 
				serie: true,
				charset: 'utf-8'
			}
		];

		// load modules
		return $ocLazyLoad.load(loaderOpts);
	};
	
	var reloadInjects = function() {

		var defer = $q.defer();

		var moduleFiles = [];
		var scriptFiles = [];

		// empty registered upgrade scripts
		upgrader.cleanup();

		// dynamicly inject upgrades (drops & dropper)
		if (core.resources && core.resources.upgradeScripts) {
			scriptFiles = scriptFiles.concat(collectResources('js', core.resources.upgradeScripts));
		}

		// dynamicly injects conditionals
		if (core.resources && core.resources.conditionals) {
			scriptFiles = scriptFiles.concat(collectResources('js', core.resources.conditionals));
		}

		// dynamicly injects admin scripts
		if (core.resources && core.resources.adminScripts) {
			moduleFiles = moduleFiles.concat(collectResources('js', core.resources.adminScripts));
		}

		// dynamicly injects admin stylesheets
		if (core.resources && core.resources.adminStylesheets) {
			moduleFiles = moduleFiles.concat(collectResources('css', core.resources.adminStylesheets));
		}

		// load scripts
		console.log('INJECTING SCRIPT FILES...', scriptFiles);
		injectScripts(scriptFiles).then(
			// success	
			function() {
				console.log('DONE INJECTING SCRIPT FILES');
				console.log('INJECTING MODULES...', moduleFiles);

				// load modules
				injectModules(moduleFiles).then(
					// success
					function() {
						console.log('DONE INJECTING MODULES');

						$timeout(function() {
							defer.resolve();
						});
					}, 
					// error
					function(err) {
						defer.reject('MODULE INJECTION ERROR', err);	
					}
				);
			}, 

			// error
			function(err) {
				defer.reject();
				console.error('SCRIPT INJECTION ERROR', err);
			}
		);

		return defer.promise;
	};

	/**
	 * Remove Stylesheets from JTL
	 */
	var removeJTLStylesheets = function() {
		if (window.jtlStylesheetsRemoved) return;
		$('head link[rel="stylesheet"]').each(function() {
			var link = $(this).attr('href');
			if (link.indexOf('templates/css/menu.css') < 0 && link.indexOf('jquery') < 0) {
				$(this).remove();
			}
		});
		window.jtlStylesheetsRemoved = true;
	};


	// we have to extend the reloading process to wait until all injects are done
	core.fn.onReloading(function() {
		return reloadInjects();
	});

	var knownRoutes = [DROPPER_ROUTES.default, DROPPER_ROUTES.backend, DROPPER_ROUTES.expired, DROPPER_ROUTES.live];
	
	// when the core is loaded, redirect to the /expired page if we have no license
	core.fn.onReload(function(coreData) {

		var requestPath = $location.path();
		$scope.ready = true;

		if (!core.dropperLicense || !core.dropperLicense.isValid) {

			$location.path(DROPPER_ROUTES.expired);

		} else {

			// we have a valid license... lets redirect to '/'
			if (requestPath === DROPPER_ROUTES.expired) requestPath = DROPPER_ROUTES.default;

			$location.path(
				knownRoutes.indexOf(requestPath) >= 0 
				? requestPath 
				: DROPPER_ROUTES.default
			);
		}
	});

	// handle redirections
	$scope.$on('$routeChangeStart', function(event, next, current) {
		
		var requestPath = $location.path();
		
		// always redirect to the expired page if no license has been found
		if (core.dropperLicense && core.dropperLicense.isValid === false) {
			$scope.$root.$evalAsync(function() {
				$location.path(DROPPER_ROUTES.expired);
			});
		}
		// redirect to / if we have a valid license and someone wants to access the /expired page
		else if (core.dropperLicense && core.dropperLicense.isValid === true) {

			// we have a valid license... lets redirect to '/'
			if (requestPath === DROPPER_ROUTES.expired) requestPath = DROPPER_ROUTES.default;

			$scope.$root.$evalAsync(function() {
				$location.path(
					knownRoutes.indexOf(requestPath) >= 0 
					? requestPath
					: DROPPER_ROUTES.default
				);
			});
		}
	});

	removeJTLStylesheets();

	// load settings, then reload the core
	core.fn.loadSettings().then(function() {
		core.fn.reload();
	});

	
}]);

angular.module('app').config(['$routeProvider', '$locationProvider', function($routeProvider, $locationProvider) {

	/*********************************************
	 Configure Routing
	 ********************************************/
	var routeCacheId = new Date().getTime();

	$routeProvider.when(DROPPER_ROUTES.expired, {
		templateUrl: window.kk_admin_url + 'js/views/expired.html.js?' + routeCacheId,
		controller: 'ExpiredCtrl'
	});

	$routeProvider.when(DROPPER_ROUTES.live, {
		templateUrl: window.kk_admin_url + 'js/views/liveview.html.js?' + routeCacheId,
		controller: 'LiveViewCtrl'
	});

	$routeProvider.when(DROPPER_ROUTES.default, {
		templateUrl: window.kk_admin_url + 'js/views/mainview.html.js?' + routeCacheId,
		controller: 'MainViewCtrl'
	});

}]);