/**
 * Environement Service
 * Provides functionality about Languages.
 * You can set the current Language here which is later used
 * All inside the UI Bindings
 */
angular.module('app').service('environment', ['core', function(core) {

	if (window.dropperEnvironment) return window.dropperEnvironment;
	
	var env = {
		setLanguage : function(lang) {
			env.language = $.extend(true, {}, lang);
		},
		
		getLanguages: function() {
			return core.languages;
		},
		
		getActiveLang: function() {
			return env.language;
		},
		
		init: function(core) {
			if (core.languages instanceof Array && core.languages.length > 0) {
		
				for(var langIdx in core.languages) {
					// get active and the default language
					if (core.languages[langIdx].cStandard == 'Y') {
						env.language = $.extend(true, {}, core.languages[langIdx]);
						env.languageDefault = $.extend(true, {}, core.languages[langIdx]);
						break;
					}
				}
		
				// no active language could be determined, use the first language available
				if (!env.language) {
					env.language = $.extend(true, {}, core.languages[0]);
				}
		
				// no default language could be determined, use the first language available
				if (!env.languageDefault) {
					env.languageDefault = $.extend(true, {}, core.languages[0]);
				}
			}
			
		}
	};

	core.fn.onReload(env.init);
	env.init(core);

	window.dropperEnvironment = env;
	return window.dropperEnvironment;
	
}]);
