/**
 * Root Scope legacy service. Used to extend the $rootScope with functions that are legacy now.
 */
angular.module('app').service('rootScopeLegacy', ['editWindow', 'utils', 'state', 'core', function(editWindow, utils, state, core) {

    var rootScopeLegacy = {
		/**
		 * Adds a new dropItem to the active drop
		 */
		addDropItem: function(drop, activate) {

			if (!drop && state.activeDrop) {
				drop = state.activeDrop;
			}

			if (typeof(activate) === 'undefined') {
				activate = true;
			}

			if (!angular.isArray(drop.dropItems)) {
				drop.dropItems = [];
			}

			var template = core.templates[drop._internal.dropId];
			var newDropItem = utils.Drops.addItem(drop.dropItems, template.itemDefaults || false, template.itemSettings, drop, 'Neuer Eintrag');

			if (activate) {
				rootScopeLegacy.activateDropItem(newDropItem);
			}

			return newDropItem;

		},
		
		/**
		 * activates a dropItem on the active drop
		 */
		activateDropItem: function(dropItem, e) {

			if (e) e.preventDefault();
			
			if (!state.activeDrop || !state.activeDrop._internal) return console.log('Could not activate drop item because active drop was not set:', dropItem);
			
			let template = core.templates[state.activeDrop._internal.dropId];

			if (!template) return console.log('Could not activate drop item because template was not found:', dropItem);
			
			editWindow.show(true, 'ListItemEditWindow', {
				item: dropItem,
				itemParent: state.activeDrop,
				itemSettings: template.itemSettings
			});
			
		},
		
		/**
		 * removes a given drop item from the active drop
		 */
		removeDropItem: function(dropItem, e) {

			if (e) {
				e.stopPropagation();
			}
			
			if (!state.activeDrop || !state.activeDrop.dropItems) return console.log('could not remove drop item because active drop was not set', dropItem);
			state.activeDrop.dropItems.splice(state.activeDrop.dropItems.indexOf(dropItem), 1);

		},
		
		/**
		 * Gets the readable name of drop without grouping in front
		 */
		ungroupLabel: function(name) {
			return utils.Drops.ungroupLabel(name);
		},
		
		/**
		 * Gets the name of the group of a drop from its name
		 */
		groupLabel: function(name) {
			return utils.Drops.groupLabel(name);
		},
		
		/**
		 * Generates a Short Code for a drop, which can be inserted into text
		 */
		getDropShortcode: function(drop) {
			return utils.Drops.getDropShortcode(drop);
		},
		
		/**
		 * Searches the given Text for drop shortcodes and returns the
		 * found drops as an array of objects like { guid: "guid-abc", found: true|false, drop: null|drop }.
		 */
		getDropsFromText: function(token) {
			return utils.Drops.getDropsFromText(token);
		},
		
		$install: function(rootScope) {
			
			// copy methods
			for(var prop in rootScopeLegacy) {
				
				// do not take $ methods
				if (!Object.prototype.hasOwnProperty.call(rootScopeLegacy, prop) || prop[0] === '$') continue;

				rootScope[prop] = rootScopeLegacy[prop];
			}
			
			// attach special getter/setter things for shimming
			Object.defineProperty(rootScope, 'activeDrop', {
			    get: function() {
			        return state.activeDrop;
			    },
			    set: function(drop) {
			    	utils.Drops.activateDrop(drop);
			    }
			});

			// attach special getter/setter things for shimming
			Object.defineProperty(rootScope, 'activeTemplate', {
			    get: function() {
			    	if (!core.templates) return null;
			    	if (!state.activeDrop || !state.activeDrop._internal || !state.activeDrop._internal.dropId) return null;
			        return core.templates[state.activeDrop._internal.dropId];
			    }
			});


		}
		
	};

	return rootScopeLegacy;

}]);
