/**
 * Message Service
 * provides basic functionality for notifiying the user
 * There are 5 types of messages: error, warning, info, success
 * All of them take a message string, an object (output as json) and optionally a delay time
 */
angular.module('app').service('message', ['$rootScope', function($rootScope) {

	// check if we were have been initialized already...
	if (window.dropperMessageService) return window.dropperMessageService;

	// build service instance	
	var msgService = {
	
		// list of shown messages
		messages: [],

		// internal message function
		_message: function(type, title, body, time, allowHTML) {
	
			var newMsg = {
				type: type,
				title: title,
				body: typeof body === 'string' ? body : JSON.stringify(body),
				time: time,
				created: new Date(),
				allowHTML: allowHTML ? true : false
			};
	
			this.messages.push(newMsg);
	
			if (time !== 0) {
				var self = this;
				setTimeout(function() {
					$rootScope.$apply(function() {
						self.removeMessage(newMsg);
					});
				}, time);
			}
	
			if (!$rootScope.$$phase) $rootScope.$apply();
		},
	
		// send error message
		error: function(title, body, time) {
			this._message('error', title, body, (typeof(time) == 'number' ? time : 0), false);
			console.log('Error: ', title, body);
		},
	
		// send warning message
		warning: function(title, body, time) {
			this._message('warning', title, body, (typeof(time) == 'number' ? time : 20000), false);
			console.log('Warning: ', title, body);
		},
	
		// send info message
		info: function(title, body, time) {
			this._message('info', title, body, (typeof(time) == 'number' ? time : 6000), false);
		},
		
		// send success message
		success: function(title, body, time) {
			this._message('success', title, body, (typeof(time) == 'number' ? time : 6000), false);
		},
	
		// send error message
		errorHTML: function(title, body, time) {
			this._message('error', title, body, (typeof(time) == 'number' ? time : 0), true);
			console.log('Error: ', title, body);
		},
	
		// send warning message
		warningHTML: function(title, body, time) {
			this._message('warning', title, body, (typeof(time) == 'number' ? time : 20000), true);
			console.log('Warning: ', title, body);
		},
	
		// send info message
		infoHTML: function(title, body, time) {
			this._message('info', title, body, (typeof(time) == 'number' ? time : 6000), true);
		},
		
		// send success message
		successHTML: function(title, body, time) {
			this._message('success', title, body, (typeof(time) == 'number' ? time : 6000), true);
		},
		
	
		// removes a given message from the queue
		removeMessage: function(msg) {
			var id = this.messages.indexOf(msg);
			if (id >= 0) this.messages.splice(id, 1);
		},
	
		// init service
		init: function() {

			// add backend errors on service initialization
			if (window.kk_backend_errors && window.kk_backend_errors.length) {
		
				for (var i=0; i<window.kk_backend_errors.length; i++) {
					var err = window.kk_backend_errors[i];
					// add error
					this.errorHTML(err.title, err.desc || '', 0);
				}
		
				// empty them, because they are added now
				window.kk_backend_errors = [];
		
			}
		}
	};
	
	// initialize
	msgService.init();
	
	// bind to global
	window.dropperMessageService = msgService;
	
	return msgService;

}]);
