/**
 * Modal Service
 * Allows to create a new Modal
 */
angular.module('app').service('modal', ['$compile', '$rootScope', '$controller', '$timeout', 'core', 'bootstrap', function($compile, $rootScope, $controller, $timeout, core, bootstrap) {

	var self = this;

	// DIRTY HACK: Select 2 won't work on modals, this will fix it hopefully
	$.fn.modal.Constructor.prototype.enforceFocus = function() {};

	this.modals = [];

	/**
	 * Closes all open modals
	 */
	this.closeAll = function() {
		for(var i=0; i<this.modals.length; i++) {
			var modal = this.modals[i];
			if (modal && angular.isFunction(modal.close)) {
				modal.close();
			}
		}
	};

	/**
	 * Shows a modal
	 */
	this.show = function(options, callback, injects) {

		// close all other open modals
		this.closeAll();

		// create isolated scope for this modal
		var modalScope = $rootScope.$new(true);
		modalScope.isStatic = false;
		modalScope.delayShow = false;

		if (self.modals.indexOf(modalScope) < 0) self.modals.push(modalScope);

		// let's run this in a timeout, so that we can prevent $digest problems
		$timeout(function() {

			// indirect call via name
			if (angular.isString(options)) {
				var modalName = options;

				options = {
					template: core.adminURL + 'js/views/modals/' + modalName.toLowerCase() + '.html.js?' + core.cacheID,
					controller: modalName + "Controller"
				};
			}

			// assign the options to our new scope
			modalScope.options = options;

			// compile the template
			var template = '<div class="modal hide fade" tabindex="-1" data-show="false" role="dialog"><ng-include src="options.template" onload="_show()"></ng-include></div>';
			var modalDom = $compile(template)(modalScope);

			// attach the close function to the scope.
			modalScope.close = function(result) {

				if (angular.isFunction(callback)) {
					callback(result);
				}

				$(modalDom).modal('hide');

				var modalIdx = self.modals.indexOf(modalScope);

				// remove the modal from our list
				if (self.modals.indexOf(modalIdx) >= 0) self.modals.splice(modalIdx, 1);

			};

			// show the modal visually
			modalScope.show = function() {

				$(modalDom).modal({
					show: true, 
					backdrop: (modalScope.noBackdrop ? false : modalScope.isStatic ? 'static' : true),
					keyboard: !modalScope.isStatic
				});

				bootstrap.rebind();

			};

			modalScope._show = function() {

				$(modalDom).appendTo('#main-view');

				// create the controller, if any given
				if (options.controller) {

					var ctrlInjects = {"$modalScope": modalScope, "$element": modalDom};

					if (angular.isObject(injects)) {
						ctrlInjects = angular.extend({}, ctrlInjects, injects);
					}

					var controller = $controller(options.controller, ctrlInjects);

				}

				if (!modalScope.delayShow) {

					// show it
					modalScope.show();

				}

				// cleanup when the modal closes
				$(modalDom).on('hidden', function() {

					modalScope.$destroy();
					$(modalDom).remove();

				});
			};
		});
	};
}]);
