/**
 * Packagemanager Service
 * Provides access to the package manager
 */
angular.module('app').service('packageManager', ['core', 'modal', function(core, modal) {
	
	var filterByType = function(list, type) {
		if (!type) return list;
		var filtered = {};
		for (var id in list) {
			var pkg = list[id];
			if (pkg && pkg.type === type) filtered[id] = pkg;
		}
		return filtered;
	};
	
	this.hasInstallablePackages = function() {
		return (Object.keys(core.packages.installable).length > 0);
	};

	this.hasUpgradeablePackages = function() {
		return (Object.keys(core.packages.upgradeable).length > 0);
	};
	
	this.hasForcedUpgrades = function() {
		return (Object.keys(this.getForcedUpgrades()).length > 0);
	};

	this.isInstallable = function(pkgId) {
		return (core.packages && core.packages.installable && typeof(core.packages.installable[pkgId]) !== 'undefined');
	};

	this.isUpgradeable = function(pkgId) {
		return (core.packages && core.packages.upgradeable && typeof(core.packages.upgradeable[pkgId]) !== 'undefined');
	};
	
	this.getInstallable = function(type) {
		return filterByType(core.packages.installable, type);
	};

	this.getUpgradeable = function(type) {
		return filterByType(core.packages.upgradeable, type);
	};

	this.getForcedUpgrades = function() {
		var forced = {};
		if (core && core.packages && core.packages.upgradeable) {
			for(var id in core.packages.upgradeable) {
				var upgrade = core.packages.upgradeable[id];
				if (upgrade.candidate && upgrade.candidate.force === true) {
					forced[id] = upgrade;
				}
			}
		}
		return forced;
	};
	
	this.getHoldback = function() {
		return core.packages.holdback;
	};
	
	this.getInstalledVersion = function(pkgId) {
		if (typeof(core.packages.installed[pkgId]) !== 'undefined') {
			return core.packages.installed[pkgId]['version'];
		}
		return false;
	};
	
	this.checkForcedUpgrades = function() {
		if (this.hasForcedUpgrades()) {
			modal.show('PackageManagerModal', null, {"overviewMode": "forcedUpgrades", "mode": "overview", "package": null});
		}
	};
	
}]);