angular.module('app').controller('ImportExportModalController', ['$modalScope', '$element', '$filter','core', 'message', 'type', 'utils', 'state', 'delta', function($modalScope, $element, $filter, core, message, type, utils, state, delta) {

	$modalScope.tab = type;
	$modalScope.core = core;
	$modalScope.cancelled = false;
	$modalScope.utils = utils;

	$modalScope.exportDrops = {
		items: [],
		settings: {
			exportStorage: true,
			exportMediafiles: true
		},
		inProgress: false
	};

	$modalScope.exportOption = {
		"description" : "",
		"optionLabel" : "_internal.name",
		"optionValue" : "_internal.guid",
		"options" : core.settings.drops
	};
	
	$modalScope.exportSettingsConfig = [
		{
			"name": "Mediendateien exportieren?",
			"description": "Sollen Bilder und andere Mediendateien, die von den ausgewählten Drops benutzt werden mit exportiert werden?",
			"options": [
				{ "label": "Ja", "value": true },	
				{ "label": "Nein", "value": false }
			],
			"type": "switch",
			"property": "exportMediafiles"
		},
		{
			"name": "Externe Daten exportieren?",
			"description": "Einige Drops speichern Inhalte extern im Drop Storage. Sollen diese Inhalte exportiert werden?",
			"options": [
				{ "label": "Ja", "value": true },	
				{ "label": "Nein", "value": false }
			],
			"type": "switch",
			"property": "exportStorage"
		}
	];

	$modalScope.importSettingsConfig = [
		{
			"name": "Aktuelle Drops löschen? (Es bleiben nur die importierten Drops übrig)",
			"description" : "Aktuelle Drops löschen? (Es bleiben nur die importierten Drops übrig)",
			"type": "switch",
			"options" : [
				{"label": "Ja", "value": true},
				{"label": "Nein", "value": false}
			],
			"property": "replace"
		},
		{
			"name": "Mediendateien importieren?",
			"description": "Sollen Bilder und andere Mediendateien, die von den ausgewählten Drops benutzt werden mit importiert werden?",
			"options": [
				{ "label": "Ja", "value": true },	
				{ "label": "Nein", "value": false }
			],
			"type": "switch",
			"property": "importMediafiles"
		},
		{
			"name": "Externe Daten importieren?",
			"description": "Einige Drops speichern Inhalte extern im Drop Storage. Sollen diese Inhalte importiert werden?",
			"options": [
				{ "label": "Ja", "value": true },	
				{ "label": "Nein", "value": false }
			],
			"type": "switch",
			"property": "importStorage"
		},
		{
			"name": "Wie sollen Sprachvariablen importiert werden?",
			"description" : "Wie sollen Sprachvariablen importiert werden?",
			"type": "switch",
			"options" : [
				{"label": "Nicht importieren", "value": "none"},
				{"label": "Eigene beibehalten", "value": "gracefull"},
				{"label": "Eigene überschreiben", "value": "override"}
			],
			"property": "langOption"
		}
	];
	
	
	$modalScope.importDrops = {
		drops: [],
		timestamp: null,
		selected: [],
		replace: false,
		importMediafiles: true,
		importStorage: true,
		lang: {},
		langOption: "none",
		inProgress: false,
		progress: ""
	};

	var jUploader = $element.find('.import-uploader');
	
	var xhrs = [];

	//////////////////////////
	// EXPORT
	//////////////////////////

	$modalScope.setTab = function(tab) {
		$modalScope.tab = tab;
	};

	$modalScope.selectAllForExport = function() {
		var res = [];
		for (var i=0; i<core.settings.drops.length; i++) {
			var guid = core.settings.drops[i]._internal.guid;
			res.push(guid);
		}
		$modalScope.exportDrops.items = res;
	};

	$modalScope.selectNoneForExport = function() {
		$modalScope.exportDrops.items = [];
	};
	
	$modalScope.doExport = function() {
		
		// do nothing when there is already an export in progress
		if ($modalScope.exportDrops.inProgress || $modalScope.cancelled) return;
		
		// do nothing on empty
		if ($modalScope.exportDrops.items.length === 0) return;
		
		$modalScope.exportDrops.inProgress = true;

		// resolve the drops
		var res = [];
		var lang = {};

		for (var i=0; i<$modalScope.exportDrops.items.length; i++) {
			var guid = $modalScope.exportDrops.items[i];
			var drop = utils.Drops.getDropByGuid(guid);
			if (!drop) continue;

			// push it
			res.push(JSON.parse(angular.toJson(drop)));

			// does the drop have lang settings?
			if (core.settings.lang && core.settings.lang[drop._internal.dropId]) {
				lang[drop._internal.dropId] = $.extend(true, {}, core.settings.lang[drop._internal.dropId]);
			}
		}

		// send guids for unloaded drops, set whole drop object for loaded ones
		res = res.map(function(drop) {
			return delta.isLoaded(drop) && delta.isChanged(drop) ? drop : drop._internal.guid;
		});

		// construct request data
		var data = angular.toJson({ 
			drops: res, 
			timestamp: new Date().getTime(),
			dropperVersion: core.dropperVersion,
			lang: lang,
			user: core.adminUser
		}, false);

		// make an ajax request that generates a .zip file
		xhrs.push($.ajax({
			type: 'POST',
			dataType: 'JSON',
			url: core.shopURL,
			data: {
				"kk_dropper_build_export": data,
				"kk_dropper_build_export_images": $modalScope.exportDrops.settings.exportMediafiles ? "true" : "false",
				"kk_dropper_build_export_storage": $modalScope.exportDrops.settings.exportStorage ? "true" : "false",
				"kk_dropper_admin_pass": core.adminSecret
			},
			success: function(res) {
				console.log(res);
				$modalScope.$apply(function() { $modalScope.exportDrops.inProgress = false });
				if (res && res.download) {
					window.location.href = res.download;
				}
			},
			error: function(err) {
				$modalScope.$apply(function() { $modalScope.exportDrops.inProgress = false });
				message.error('Export konnte nicht erstellt werden (' + err.status + ')' + (err.statusText ? ': ' + err.statusText : ''), err.responseText);
			}
		}));
		
	};

	//////////////////////////
	// IMPORT
	//////////////////////////

	var archive = null;
	var zip = new JSZip();

	$modalScope.showUploader = function() {
		jUploader.click();
	};


	// read the file when there is one selected!
	jUploader.change(function() {

		var file = $(this).get(0).files[0];

		if (file) {

			try {
			
				if (!file.name) throw "Dateiname konnte nicht gelesen werden.";

				// get the file extension
				var ext = file.name.split('.').pop().toLowerCase();

				// valid?
				if (['zip', 'json'].indexOf(ext) < 0) {
					throw "Dateiendung nicht unterstützt: '" + ext + "'";
				}

				// create a file reader
				var reader = new FileReader();
	
				// once loaded try to extract the file as .zip
				reader.onload = function(e) {
	
					try {
	
						// get the binary data as string
						var data = e.target.result;
	
						// old plain .json format?
						if (ext === 'json') {
							var data = angular.fromJson(e.target.result);
							$modalScope.$apply(function() {
								$modalScope.setImportData(data);
							});
						} 
	
						// new .zip file format
						else if (ext === 'zip') {
	
							// read the data as zip and extract the index
							zip.loadAsync(data).then(function(zipReader) {
		
								// save the reference
								archive = zipReader;
		
								// read the drop export data
								var dataFile = archive.file('data.json');
								var indexFile = archive.file('index.json');
		
								if (dataFile === null || indexFile === null) {
									$modalScope.close();
									return message.error('Backup hat das falsche Format.', 'Das Backup konnte leider nicht gelesen werden oder liegt im falschen Format vor');
								}
		
								dataFile.async("string").then(function(data) {
									// set the parsed data
									$modalScope.setImportData(JSON.parse(data));
								}).then(function() {
									// read the index
									indexFile.async("string").then(function(data) {
										// set the parsed data
										$modalScope.importIndex = JSON.parse(data); 
										$modalScope.$apply();
									});
								});
							});						
						}
						
					} catch (err) {
						message.error("Backup konnte nicht geladen werden!", err.message ? err.message : err);
					}

				};
				
				// error reading the file
				reader.onerror = function(e) {
					message.error("Backup konnte nicht gelesen werden!", "Eventuell ist die Datei korrupt oder zu groß.");
				};

				// read as utf-8 text if json format (OLD)
				if (ext === 'json') reader.readAsText(file, "UTF-8");
				
				// read the file as binary when it is a zip
				if (ext === 'zip') reader.readAsBinaryString(file);

			} catch(err) {
				message.error("Backup konnte nicht geladen werden!", err.message ? err.message : err);
			}
		}
	});

	$modalScope.setImportData = function(data) {

		if (data.drops) {

			var included = [];

			for (var i=0; i<data.drops.length; i++) {
				var drop = data.drops[i];
				var importState = $modalScope.getImportState(drop);
				if ($modalScope.canImport(drop) && importState !== "same") {
					included.push(drop);
				}
			}

			$modalScope.importDrops.timestamp = data.timestamp;
			$modalScope.importDrops.user = data.user;
			$modalScope.importDrops.drops = data.drops;
			$modalScope.importDrops.lang = data.lang || {};
			$modalScope.importDrops.selected = included;
		}
	};

	$modalScope.importToggle = function(drop) {
		var idx = $modalScope.importDrops.selected.indexOf(drop);
		if (idx >= 0) {
			// drop is selected, remove it from the list
			$modalScope.importDrops.selected.splice(idx, 1);
		} else {
			$modalScope.importDrops.selected.push(drop);
		}
	};

	$modalScope.isImportSelected = function(drop) {
		return ($modalScope.importDrops.selected.indexOf(drop) >= 0);
	};

	$modalScope.getImportState = function(drop) {

		var own = utils.Drops.getDropByGuid(drop._internal.guid);

		// new one?
		if (!own) return "new";

		// one has revision, the other not
		if (drop._internal.revision && !own._internal.revision) return "update_newer";
		if (own._internal.revision && !drop._internal.revision) return "update_older";
		
		// both have the same revision
		if (drop._internal.revision && own._internal.revision) {
			if (drop._internal.revision.id === own._internal.revision.id) return "same";
			if (drop._internal.revision.id < own._internal.revision.id) return "update_older";
			if (drop._internal.revision.id > own._internal.revision.id) return "update_newer";
		}
		
		// LEGACY: both not revisioned? check if it is the same by JSON!
		var ownJSON = angular.toJson(own);
		var dropJSON = angular.toJson(drop);

		return (ownJSON === dropJSON ? "same" : "update");

	};

	$modalScope.canImport = function(drop) {
		var dropId = drop._internal.dropId;
		return utils.Drops.canAddDrop(dropId);
	};

	$modalScope.getImportDate = function() {
		var date = new Date($modalScope.importDrops.timestamp);
		return $filter('date')(date, 'dd. MMMM, HH:mm');
	};

	$modalScope.selectAllForImport = function() {
		var res = [];
		for (var i=0; i<$modalScope.importDrops.drops.length; i++) {
			var drop = $modalScope.importDrops.drops[i];
			res.push(drop);
		}
		$modalScope.importDrops.selected = res;
	};

	$modalScope.selectNoneForImport = function() {
		$modalScope.importDrops.selected = [];
	};

	$modalScope.extractAndUploadImportFiles = function(archive, cb) {

		// no archive? seems to don't have the zip file format!
		if (!archive) {
			return cb(null);
		}

		// max size for the payload to be sent with one request
		var BATCH_MAX_SIZE = 3 * 1024*1024;
		
		// group a maximum of these files into one request, because we do not want 
		// to produce to many POST parameters (which are also limited, see 'max_input_vars')
		var BATCH_MAX_FILES = 50;

		// limit of how much requests can be done in parallel
		var BATCH_CONCURRENCY = 3;

		// init
		$modalScope.importDrops.progress = 'Datenaufbereitung...';
		var selectedDrops = $modalScope.importDrops.selected;
		var index = $modalScope.importIndex;
		
		// list of files to keep
		var files = [];

		for (var i=0; i<selectedDrops.length; i++) {

			var drop = selectedDrops[i];
			var guid = drop._internal.guid;
			var dropIndex = index[guid];

			// list of media files to import
			var dropMediaFiles = [];
			
			if ($modalScope.importDrops.importMediafiles) {
				dropMediaFiles = dropIndex.mediafiles.map(function(file) {
					return { 
						path: 'mediafiles/' + file,
						type: 'mediafiles'
					};
				});
			}
			
			// list of storage files to import
			var dropStorageFiles = [];
			
			if ($modalScope.importDrops.importStorage) {
				dropStorageFiles = dropIndex.storage.map(function(file) {
					return { 
						path: 'storage/' + file,
						type: 'storage'
					};
				});
			}

			// list of all files to import
			files = files.concat(dropMediaFiles, dropStorageFiles);
		}
		
		// filter duplicates
		var paths = [];
		var uploadFiles = [];

		// build collection of files to send
		files.forEach(function(file) {
			if (paths.indexOf(file.path) < 0) {
				paths.push(file.path);
				// enhance with date and size
				var zipFile = archive.file(file.path);
				file.date = Math.floor(zipFile.date.getTime() / 1000);
				file.size = zipFile._data.uncompressedSize || 0;
				uploadFiles.push(file);
			}
		});
		
		// no files to upload? return here
		if (!uploadFiles.length && cb) return cb(null);

		// sort them by size, beginning with the smallest one
		uploadFiles = uploadFiles.sort(function(a, b) {
			return a.size - b.size;	
		});

		// build groups of files, that add up until the size limit is reached
		var runningSize = 0;
		var runningGroup = [];
		var batchFileGroups = [runningGroup];

		uploadFiles.forEach(function(file) {

			// new group?
			if (runningGroup.length && runningSize + file.size >= BATCH_MAX_SIZE || runningGroup.length >= BATCH_MAX_FILES) {
				runningGroup = [];
				runningSize = 0;
				batchFileGroups.push(runningGroup);
			}

			runningGroup.push(file);
			runningSize = runningSize + file.size;

		});

		var sentFiles = 0;
		var calledRequests = 0;
		var totalFiles = uploadFiles.length;
		var totalGroups = batchFileGroups.length;

		// extracts the contents for a group of files
		var extractGroupContents = function(group, idx, cb) {

			if (idx >= group.length) return cb();

			var file = group[idx];
			var zipFile = archive.file(file.path);

			zipFile.async("base64").then(function(content) {
				file.content = content;
				extractGroupContents(group, idx+1, cb);
			});
			
		};

		// sends a batch of files to the server
		var sendBatch = function(batchSize) {

			var batchGroups = batchFileGroups.splice(0, batchSize);
			var batchCount = 0;

			batchGroups.forEach(function(fileGroup) {

				// extract contents for this group
				extractGroupContents(fileGroup, 0, function() {

					// update progress
					$modalScope.$apply(function() {
						$modalScope.importDrops.progress = 'Sende Daten: ' + Math.floor(100 * calledRequests / totalGroups) + '%';	
					});

					// don't make any new requests when the user has cancelled
					if ($modalScope.cancelled) return;

					// send this batch of files over
					xhrs.push($.ajax({
						type: 'POST',
						url: core.shopURL,
						dataType: 'JSON',
						data: {
							"kk_dropper_import_backup_files": fileGroup,
							"kk_dropper_admin_pass": core.adminSecret
						},
	
						// upload successfull
						success: function(res) {
	
							sentFiles+=fileGroup.length;
							calledRequests++;
							batchCount++;

							// are we done?
							if (calledRequests >= totalGroups) { 
								$modalScope.$apply(function() { 
									$modalScope.importDrops.inProgress = false; 
									if (cb) cb(null);
								});
							}
	
							$modalScope.$apply(function() {
								$modalScope.importDrops.progress = 'Sende Daten: ' + Math.floor(100 * calledRequests / totalGroups) + '%';
							});
							
							// send the next file batch because this one is finished!
							if (batchFileGroups.length) {
								sendBatch(1);
							}
						},
	
						// upload failed
						error: function(err) {
	
							calledRequests++;
							batchCount++;

							// send the next file because this one is finished!
							if (batchFileGroups.length) {
								sendBatch(1);
							}

							// are we done?
							if (calledRequests >= totalGroups) {
								$modalScope.$apply(function() { 
									$modalScope.importDrops.inProgress = false; 
									if (cb) cb(null);
								});
							}
						}
					}));					

				});
			

			});

		};
		
		// send 7 files in parrallel
		sendBatch(BATCH_CONCURRENCY);

	};

	$modalScope.doImport = function() {

		// already importing?
		if ($modalScope.importDrops.inProgress) return;
		
		// reset progress label
		$modalScope.importDrops.progress = "";

		// set us working
		$modalScope.importDrops.inProgress = true;

		// no drops? no game
		if ($modalScope.importDrops.selected.length === 0) return;

		// upload images and storage to the server
		$modalScope.extractAndUploadImportFiles(archive, function(err) {

			// do nothing in case of import errors
			if (err) return;

			var replacedItems = 0;
			var newItems = 0;
	
			// empty the drops from settings
			if ($modalScope.importDrops.replace === true) {
				delta.resetChanged();
				delta.setAllRemoved();
				core.settings.drops = [];
			}
	
			for (var i=0; i<$modalScope.importDrops.selected.length; i++) {
	
				var importDrop = $modalScope.importDrops.selected[i];
				var importTemplate = importDrop._internal.dropId;
	
				// update the revision so that we ensure that there are no conflicts when saving
				var revision = {
					id: -1,		// tells the server that this drop is force imported
					user: importDrop._internal.revision && importDrop._internal.revision.user ? importDrop._internal.revision.user : 'Unbekannter Nutzer',
					timestamp: importDrop._internal.revision && importDrop._internal.revision.timestamp ? importDrop._internal.revision.timestamp : null
				};
				
				importDrop._internal.revision = revision;

				// get a reference to the original drop (if existant)
				var orig = utils.Drops.getDropByGuid(importDrop._internal.guid);

				// replace it!
				if (orig) {
					var origIdx = core.settings.drops.indexOf(orig);
					if (origIdx >= 0) {
						core.settings.drops[origIdx] = importDrop;
						replacedItems ++;
					}
				} 
				// add it to the list
				else {
					
					core.settings.drops.push(importDrop);
					newItems ++;
				}

				// announce change and we already have the full version here
				delta.setChanged(importDrop);

				// merge lang setting
				if ($modalScope.importDrops.langOption !== "none" && $modalScope.importDrops.lang && $modalScope.importDrops.lang[importTemplate]) {

					if (!core.settings.lang) core.settings.lang = {};
					if (!core.settings.lang[importTemplate]) core.settings.lang[importTemplate] = {};
	
					switch($modalScope.importDrops.langOption) {
						case "override":
							core.settings.lang[importTemplate] = $.extend(true, core.settings.lang[importTemplate], $modalScope.importDrops.lang[importTemplate]);
						break;
						case "gracefull":
							core.settings.lang[importTemplate] = $.extend(true, $modalScope.importDrops.lang[importTemplate], core.settings.lang[importTemplate]);
						break;
					}
				}
			}

			if (replacedItems > 0 || newItems > 0) {
				state.verifyChanges();
			}

			// done!
			$modalScope.importDrops.inProgress = false;
	
			message.success('Import abgeschlossen', 'Es wurden ' + replacedItems + ' Drops durch die importierte Version ersetzt und ' + newItems + ' neue Drops hinzugefügt', 6000);
			$modalScope.close();
			
		});

	};

	$modalScope.abort = function() {

		// cancel all xhrs
		xhrs.forEach(function(xhr) {
			if (xhr && typeof(xhr.abort) === 'function') xhr.abort();
		});
		
		// save that we have cancelled, so that we do not start any new requests
		$modalScope.cancelled = true;
		
		// close the modal
		$modalScope.close();
	};

}]);
