angular.module('app').controller('MediaGalleryModalController', ['$modalScope','$element', '$timeout', 'core', 'state', 'delta', 'utils', 'datasource', 'message', 'initial', 'selectable', function($modalScope, $element, $timeout, core, state, delta, utils, datasource, message, initial, selectable) {

	$modalScope.utils = utils;
	$modalScope.file = {url:""};
	$modalScope.searchFiles = [];
	$modalScope.search = {
		filename: ""
	};
	
	$modalScope.state = {
		state: "browse",
		previewstate: "references",
		deleteMode: false,
		selectmode: selectable,
		nofiles: false
	};

	if (initial) {
		$modalScope.selectedfile = initial;
	}

	$modalScope.paginate = {};

	$modalScope.$watch('mediafiles', function(files) {
		$modalScope.updateSearchFilter();
	});

	$modalScope.$watch('search.filename', function(files) {
		$modalScope.updateSearchFilter();
	});

	$modalScope.updateSearchFilter = function() {
		$modalScope.searchFiles.splice(0, $modalScope.searchFiles.length);
		$modalScope.paginate.currentPage = 0;

		if (!$modalScope.mediafiles || !$modalScope.mediafiles.length) return;

		// add all mediafiles to the searchFiles on empty search
		if ($modalScope.search.filename === "") {
			$modalScope.searchFiles.push.apply($modalScope.searchFiles, $modalScope.mediafiles);
			return;
		}

		for (var i=0; i<$modalScope.mediafiles.length; i++) {
			var file = $modalScope.mediafiles[i];
			if (file.filename.toLocaleLowerCase().indexOf($modalScope.search.filename.toLocaleLowerCase()) >= 0) {
				$modalScope.searchFiles.push(file);
			}
		}
	};

	var previewTimeout;

	new qq.FileUploader({
		element: $element.find('.uploader')[0],
		inputName: 'kk_dropper_media_upload',
		uploadButtonText: 'Datei hochladen',
		cancelButtonText: 'Abbrechen',
		failUploadText: 'Upload fehlgeschlagen',
		dragText: 'Ziehe Dateien zum Upload hier hin!',
		template: '<div class="qq-uploader"><div class="qq-upload-button btn btn-primary p100"><small><i class="icon-upload icon-white"></i> {uploadButtonText}</small></div><ul class="qq-upload-list"></ul></div>',
		disableDefaultDropzone: true,
		action: core.uploadURL,
		customHeaders: {
			'X-KK-Dropper-Admin-Pass': core.adminSecret
		},
		onComplete: function(id, fileName, response) {
			if (!response.error) {
				$('.qq-upload-list', $element).empty();
				$modalScope.$apply(function() {
					console.log('upload complete: ', response);
					$modalScope.loadFiles();
				});
			}
		},
		onError: function(id, fileName, err) {
			message.error("Datei konnte nicht hochgeladen werden!", err);
		}
	});

	$modalScope.toggleBrowse = function() {
		$modalScope.state.state='browse';
	};

	$modalScope.toggleUpload = function() {
		$modalScope.state.state='upload';
		$timeout(function() {
			$element.find('input[type="file"]').click();
		});
	};

	$modalScope.toggleSearch = function() {
		$modalScope.state.state='search';
		$timeout(function() {
			$element.find('input.search').focus();
		});
	};

	$modalScope.loadFiles = function() {
		datasource.get('MediaFiles', function(images) {
			if (!images || images.length === 0) {
				$modalScope.mediafiles = [];
				$modalScope.state.nofiles = true;
			}
			else {
				$modalScope.mediafiles = images;
				$modalScope.state.nofiles = false;

				// open up initial file, if we have one
				if (initial) {
					for (var i=0; i<images.length; i++) {
						if (images[i].path === initial) {
							$modalScope.preview(images[i]);
							break;
						}
					}
				}
			}

			$modalScope.$apply();
		}, true);
	};

	$modalScope.getFileReferences = function(file) {

		if (!file || !file.path) {
			console.log("oups, no file or path", file);
			return [];
		}
		
		// generate a search buffer from loaded drops
		if (!$modalScope.searchBuffer) {
			$modalScope.searchBuffer = [];
			var drops = core.settings.drops;
			
			for (var i=0; i<drops.length; i++) {
				var drop = drops[i];
				if (!delta.isLoaded(drop)) continue;
				$modalScope.searchBuffer.push({ drop: drop, search: utils.Drops.dropToSearch(drop) });
			}
		}

		var references = [];

		// push refrences of loaded drops
		for (var j=0; j<$modalScope.searchBuffer.length; j++) {
			var search = $modalScope.searchBuffer[j];
			if (search.search.indexOf(file.path) >= 0) {
				references.push(search.drop);
			}
		}
		
		// push references of not loaded drops
		if (core.mediaReferences[file.path]) {
			var refs = core.mediaReferences[file.path];
			for (var j=0; j<refs.length; j++) {
				var drop = utils.Drops.getDropByGuid(refs[j]);
				if (delta.isLoaded(drop)) continue;
				if (drop && references.indexOf(drop) < 0) {
					references.push(drop);
				}
			}
		}
		
		return references;
		
	};

	$modalScope.preview = function(file) {
		var prevpane = $element.find('.media-gallery-preview');
		var imgwrap = $element.find('.media-prev-wrap');
		var img = imgwrap.find('img');
		$timeout.cancel(previewTimeout);

		// look where this might be referenced
		$modalScope.fileReferences = $modalScope.getFileReferences(file);

		// no image?
		if (!file.isImage) {
			$modalScope.previewOpen = true;
			$modalScope.activefile = file;
			setTimeout(function() {
				var contentHeight = $element.outerHeight() - $element.find('.file-info').outerHeight() - parseInt(prevpane.css('margin-top'), 10);

				// set wrap dimensions
				imgwrap.css({
					'height' : contentHeight + 'px',
					'width' : $element.width() + 'px',
					'display' : 'block'
				});
			});

			return;
		}

		// already loaded
		if ($modalScope.activefile && file.url === $modalScope.activefile.path) {
			$modalScope.previewOpen = true;
			return;
		}

		// not yet loaded
		img.on('load', function() {
			$(this).off('load');

			$modalScope.$apply(function() {
				$modalScope.previewOpen = true;
			});

			var contentHeight = $element.outerHeight() - $element.find('.file-info').outerHeight() - parseInt(prevpane.css('margin-top'), 10);

			// set wrap dimensions
			imgwrap.css({
				'height' : contentHeight + 'px',
				'width' : $element.width() + 'px',
				'display' : 'block'
			});

			// if the image doesn't fit, make it fit
			if (file.width > imgwrap.width() || file.height > imgwrap.height()) {
				img.css({
					width: '100%',
					height: ''
				});

				if (img.outerHeight() > imgwrap.height()) {
					img.css({
						width: '',
						height: imgwrap.height()
					});
				}
			// image fits, dont set sizes
			} else {
				img.css({
					width: '',
					height: ''
				});
			}

			// remove temporary display
			imgwrap.css({
				'display' : ''
			});
		});

		$modalScope.activefile = file;
	};

	$modalScope.exitPreview = function() {
		$modalScope.previewOpen = false;
		$modalScope.state.deleteMode = false;

		previewTimeout = $timeout(function() {
			$modalScope.activefile = null;
			$element.find('.media-prev-wrap  img').attr("src", "");
		}, 300);
	};

	$modalScope.getThumb = function(file, size) {
		// no thumb url? return original file to preview
		if (file && file.isImage && !file.thumb) return file.path;

		if (file && file.thumb && file.isImage) {
			return file.thumb + "&width=" + size;
		}
		return core.shopURL + "gfx/keinBild.gif";
	};

	$modalScope.showCopyTag = function() {
		if ($modalScope.activefile) prompt('Drücke STRG+C um den Tag in die Zwischenablage zu kopieren', '<img src="' + $modalScope.activefile.path + '" width="' + $modalScope.activefile.width + '" height="' + $modalScope.activefile.height + '" alt=""/>');
	};

	$modalScope.isSelected = function(file) {
		if (selectable && file && $modalScope.selectedfile && file.path === $modalScope.selectedfile) {
			return true;
		}
		return false;
	};


	$modalScope.select = function(file) {
		if (!file || !selectable) {
			$modalScope.exitPreview();
			return;	
		} 

		$modalScope.selectedfile = file.path;

		$timeout(function() {
			$modalScope.close(file);
		}, 400);

	};

	// deletes the given file
	$modalScope.removeMediaItem = function(file, doublecheck) {
		if (doublecheck && !confirm("'" + file.filename + "wirklich löschen, ' obwohl sie von " + $modalScope.fileReferences.length + " Drops referenziert wird?")) {
			return;
		}
		// delete it!
		$.ajax({
			type: 'POST',
			url: core.shopURL,
			data: { "kk_dropper_admin_pass": core.adminSecret, "kk_dropper_remove_media_file": file.filename },
			dataType: 'json',
			success: function(res) {
				if (res && res.file && res.success) {
					message.success("Datei gelöscht", "Die Mediendatei '" + file.filename + "' wurde erfolgreich gelöscht");
				}
				$modalScope.loadFiles();
			},
			error: function(err) {
				message.error("Ein Fehler beim Löschen der Mediendatei ist aufgetreten", err);
			}
		});
		$modalScope.exitPreview();
	};

	// watch for file uploads. when changed, reload the library
	$modalScope.$watch("file", function() {
		if ($modalScope.file && $modalScope.file.url && $modalScope.file.url !== "") {
			$modalScope.file.url = "";
			$modalScope.loadFiles();
		}
	}, true);

	// init
	$modalScope.loadFiles();

}]);