angular.module('app').controller('UrlPickerModalController', ['$modalScope', 'urlPickerModel', 'urlPickerConfig', 'typeConfig', function($modalScope, urlPickerModel, urlPickerConfig, typeConfig){

	$modalScope.typeConfig = typeConfig;
	$modalScope.urlPickerConfig = urlPickerConfig;

	/**
	 * converts 
	 * {__url:true, type: "vendor", value: "123", title: "xxx", openExternal: true } to
	 * { type: "vendor", vendorId: "123", title: "xxx", openExternal: true }
	 */
	$modalScope.modelToSetting = function(model){

		var setting = {
			openExternal: false
		};
		
		if(!model) return setting;

		if(model.type){
			setting[model.type + 'Id'] = model.value;
		}

		setting.title = model.title;
		setting.type = model.type;
		setting.openExternal = model.openExternal || false;
		return setting;
	};


	/**
	 * converts 
	 * { type: "vendor", vendorId: "123", title: "xxx" } to
	 * {__url:true, type: "vendor", value: "123", title: "xxx"}
	 */
	$modalScope.settingToModel = function(setting){

		var model = {};
		
		if(!setting) return model;

		if(setting.type){
			model.value = setting[setting.type + 'Id'];
		}

		model.type = setting.type;
		model.title = setting.title;
		model.openExternal = setting.openExternal || false;
		model['_url'] = true;

		return model;
	};

	$modalScope.isValidLink = function(settings){
		if(!settings) return false;
		var type = settings.type;
		if(!type) return false;
		var value = settings[type + 'Id'];
		if(!value || typeof(value) === 'undefined' || value === null || (angular.isString(value) && !value.length)){
			return false;
		}
		return true;
	};

	$modalScope.applySettings = function(settings){
		if($modalScope.isValidLink(settings)){
			$modalScope.close($modalScope.settingToModel(settings));
		}
	};

	// defaults
	$modalScope.pickerSettings = $.extend({
		type: "Manual"
	}, $modalScope.modelToSetting(urlPickerModel));
}]);