conditional['CartState'] = {
	description: 'Erlaubt das Filtern über bestimmte Zustände des Warenkorbs (Session)',
	name: 'Warenkorbzustand',

	section: "Session",
	sort: 20,
	
	config: [
		{
			description: 'Um welchen Zustand geht es?',
			type: 'select',
			options: [
				{ label: "Warenkorb ist gefüllt", value: 'cartNotEmpty' },
				{ label: "Warenkorb ist leer", value: 'cartEmpty' },
				{ label: "Enthält Artikel (Auswahl)", value: 'cartContainsArticles' },
				{ label: 'Bestellsumme (von - bis)', value: 'cartOrderSumRange' },
				{ label: 'Anzahl Artikel (von - bis)', value: 'cartProductCountRange' },
				{ label: 'Anzahl Artikelpositionen (von - bis)', value: 'cartPositionCountRange' }
			],
			property: 'type'
		},
		{
			name: "Artikel",
			description: "Artikel die sich gemeinsam im Warenkorb befinden müssen",
			type: "multiselect",
			options: "Articles",
			property: "articles",
			condition: "self.type === 'cartContainsArticles'"
		},
		{
			name: "Von",
			description: "Minimale Bestellsumme",
			type: "text",
			append: "€",
			property: "cartOrderSumMin",
			condition: "self.type === 'cartOrderSumRange'"
		},
		{
			name: "Bis",
			description: "Maximale Bestellsumme",
			type: "text",
			append: "€",
			property: "cartOrderSumMax",
			condition: "self.type === 'cartOrderSumRange'"
		},
		{
			name: "Von",
			description: "Minimale Anzahl",
			type: "text",
			property: "cartProductCountMin",
			condition: "self.type === 'cartProductCountRange'"
		},
		{
			name: "Bis",
			description: "Maximale Anzahl",
			type: "text",
			property: "cartProductCountMax",
			condition: "self.type === 'cartProductCountRange'"
		},
		{
			name: "Von",
			description: "Minimale Anzahl",
			type: "text",
			property: "cartPositionCountMin",
			condition: "self.type === 'cartPositionCountRange'"
		},
		{
			name: "Bis",
			description: "Maximale Anzahl",
			type: "text",
			property: "cartPositionCountMax",
			condition: "self.type === 'cartPositionCountRange'"
		}
	],
	evaluateToSmarty: function(filter) {

		if(!filter.type) return false;

		switch(filter.type){
			case 'cartNotEmpty':
				return '!empty($smarty.session.Warenkorb->PositionenArr)';
			break;

			case 'cartEmpty':
				return 'empty($smarty.session.Warenkorb->PositionenArr)';
			break;

			case 'cartContainsArticles':
				if (!angular.isArray(filter.articles) || !filter.articles.length) return false;
				var ids = '[' + filter.articles.join(',') + ']';
				return '"' + ids + '"|is_product_in_cart';
			break;

			case 'cartOrderSumRange':
				var lower = filter.cartOrderSumMin ? "$smarty.session.Warenkorb->gibGesamtsummeWaren(!$NettoPreise) >= " + filter.cartOrderSumMin : '';
				var upper = filter.cartOrderSumMax ? "$smarty.session.Warenkorb->gibGesamtsummeWaren(!$NettoPreise) <= " + filter.cartOrderSumMax : '';
				return 'isset($smarty.session.Warenkorb) && ' + lower + (lower && upper ? ' && ' : '') + upper;
			break;

			case 'cartProductCountRange':
				var lower = filter.cartProductCountMin ? "$WarenkorbArtikelanzahl >= " + filter.cartProductCountMin : '';
				var upper = filter.cartProductCountMax ? "$WarenkorbArtikelanzahl <= " + filter.cartProductCountMax : '';
				return 'isset($WarenkorbArtikelanzahl) && ' + lower + (lower && upper ? ' && ' : '') + upper;
			break;

			case 'cartPositionCountRange':
				var lower = filter.cartPositionCountMin ? "$WarenkorbArtikelPositionenanzahl >= " + filter.cartPositionCountMin : '';
				var upper = filter.cartPositionCountMax ? "$WarenkorbArtikelPositionenanzahl <= " + filter.cartPositionCountMax : '';
				return 'isset($WarenkorbArtikelPositionenanzahl) && ' + lower + (lower && upper ? ' && ' : '') + upper;
			break;

		}
		return false;
	}
};